## ------------------------------------------------------------------------
rm(list = ls()) # Cleans environment

## ---- message=FALSE------------------------------------------------------
# You can install packages in case they are not installed already. 
# Just un-comment the next line in the script and run it. 
# install.packages(c("tidyverse", "likert", "gridExtra"));
# install.packages("jsonlite", type="binary")
library(tidyverse)
library(likert)
library(gridExtra)


## ------------------------------------------------------------------------
survey_answers <- read.csv2("complete_survey_responses.csv", header = T)

# We also create a graphical object that has the default theme for ggplot throughout this script.
PLOT_THEME <- theme_minimal(base_size = 14) + 
        theme(plot.title = element_text(size = 14, face = "bold", hjust = 0.5),
        panel.grid = element_blank(), axis.line = element_line(size = 1), axis.ticks = element_line(size = 1),
        strip.background = element_rect(colour = "white", fill = "aliceblue"), legend.position = "bottom")


## ------------------------------------------------------------------------
DEFINITION_QUESTIONS_COL <- 10:41

#Selects columns and turns values into factors to be used as likert scales.
bot_definitions_df <- select(survey_answers,DEFINITION_QUESTIONS_COL) %>% mutate_all(factor)
colnames(bot_definitions_df) <- paste("Q",1:ncol(bot_definitions_df), sep="") #Replaces questions text for IDs

print(paste("Total number of 'bot definition' questions:",ncol(bot_definitions_df),
            "; Total number of participants:",nrow(bot_definitions_df)))


## ----fig1, fig.width = 6, fig.height = 6, fig.align = 'center'-----------
likert_bot_definition <- likert(bot_definitions_df)
fig1 <- plot(likert_bot_definition) + ggtitle("Is the following tool a bot?") + PLOT_THEME
ggsave("diverging_plot_botdef.pdf", fig1, width = 18, height = 20, units = "cm")

## ------------------------------------------------------------------------
persona_map_df <- read.csv2("persona_mapping.csv", header = T)
persona_map_df


## ------------------------------------------------------------------------
#Questions that increase each participant's persona scores
qid_alex_rewards    <- persona_map_df %>% filter(Alex == "x") %>% pull(QuestionID) %>% as.character
qid_charlie_rewards <- persona_map_df %>% filter(Charlie == "x") %>% pull(QuestionID) %>% as.character
qid_sam_rewards     <- persona_map_df %>% filter(Sam == "x")  %>% pull(QuestionID) %>% as.character

#Questions that reduce from each participant's persona scores
qid_alex_punish   <- persona_map_df %>% filter(Alex == "") %>% pull(QuestionID) %>% as.character
qid_charlie_punish<- persona_map_df %>% filter(Charlie == "") %>% pull(QuestionID) %>% as.character
qid_sam_punish    <- persona_map_df %>% filter(Sam == "") %>% pull(QuestionID) %>% as.character


## ------------------------------------------------------------------------
# Function that convert the likert scores from [1,5] to [-2,2]
# Recall that before we converted the numerical values into factors.
convert_to_reward <- function(x) {case_when(
  x ==1 ~ -2,
  x ==2 ~ -1,
  x ==3 ~ 0,
  x ==4 ~ 1,
  x ==5 ~ 2)}

#Data frames for the reward scores. Adds a Sum column with all scores
alex_reward_df <- bot_definitions_df %>% select(!!qid_alex_rewards) %>% 
  mutate_all(.funs = convert_to_reward) %>% mutate(Sum = rowSums(., na.rm=T))

charlie_reward_df <- bot_definitions_df %>% select(!!qid_charlie_rewards) %>% 
  mutate_all(.funs = convert_to_reward) %>% mutate(Sum = rowSums(., na.rm=T))

sam_reward_df <- bot_definitions_df %>% select(!!qid_sam_rewards) %>% 
  mutate_all(.funs = convert_to_reward) %>% mutate(Sum = rowSums(., na.rm=T))


## ------------------------------------------------------------------------
# Function that convert the likert scores from [1,5] to [2,-2]
convert_to_punish <- function(x) {case_when(
  x ==1 ~ 2,
  x ==2 ~ 1,
  x ==3 ~ 0,
  x ==4 ~ -1,
  x ==5 ~ -2)}

#Data frames for the punishing scores.
alex_punish_df   <- bot_definitions_df %>% select(!!qid_alex_punish) %>% 
  mutate_all(.funs = convert_to_punish) %>% mutate(Sum = rowSums(., na.rm=T))

charlie_punish_df<- bot_definitions_df %>% select(!!qid_charlie_punish) %>% 
  mutate_all(.funs = convert_to_punish) %>% mutate(Sum = rowSums(., na.rm=T))

sam_punish_df    <- bot_definitions_df %>% select(!!qid_sam_punish) %>% 
  mutate_all(.funs = convert_to_punish) %>% mutate(Sum = rowSums(., na.rm=T))


## ------------------------------------------------------------------------
count_neutral_values <- function(row_values){
  #Participant either "Skiped" (NA), or "Unclear" (score == 3)
  neutral_values <- which(is.na(row_values) | row_values == 3)
  return(length(neutral_values))
}

aggregate_neutral_scores <- function(persona_df) {
  modified_df <- persona_df %>%
    rowwise() %>% do(row_values = as_tibble(.)) %>%
    mutate(NeutralValues = count_neutral_values(row_values)) %>% unnest(cols = c(row_values))
  return(modified_df)
}

neutral_scores <- aggregate_neutral_scores(bot_definitions_df) %>% select(NeutralValues)


## ------------------------------------------------------------------------
num_of_questions <- ncol(bot_definitions_df)
charlie_scores<- (charlie_reward_df$Sum + charlie_punish_df$Sum) / (num_of_questions - neutral_scores$NeutralValues)
sam_scores    <- (sam_reward_df$Sum + sam_punish_df$Sum) / (num_of_questions - neutral_scores$NeutralValues)
alex_scores   <- (alex_reward_df$Sum + alex_punish_df$Sum) / (num_of_questions - neutral_scores$NeutralValues)

scores_df <- data.frame(Charlie = charlie_scores, Alex = alex_scores, Sam = sam_scores)
scores_df


## ------------------------------------------------------------------------
# Constant values used for plotting and filtering of data.
BOT_USAGE_COLUMN    <- 43
BOT_EXPERIENCE_LABEL<- "Bot Experience"
NON_BOT_USER_LABEL  <- "No Bots Used"

#Copy the scores_df into a new datafram to keep this part of the analysis self-contained
personas_df <- scores_df
  
personas_df <- cbind(BotUsers = survey_answers[,BOT_USAGE_COLUMN], scores_df);
personas_df <- personas_df %>% mutate_at(.vars = c("BotUsers"), .funs = function(x){if_else(x == 1,BOT_EXPERIENCE_LABEL,NON_BOT_USER_LABEL)})


## ------------------------------------------------------------------------
# Function to classify a participant (row) based on its scores.
identify_main_persona <- function(participant){
  # Set threshold for participants belonging to no persona
  threshold <- 0 
  hasPersona <- participant$Charlie > threshold | participant$Alex > threshold | participant$Charlie > threshold
  main_persona <- if_else(hasPersona, colnames(participant)[max.col(participant)], "None")
  return(main_persona)
}

main_personas_df <- personas_df %>% select(-BotUsers) %>% #Temporarily drop the BotUsers column
  rowwise() %>% do(participant = as_tibble(.)) %>%
  mutate(MainPersona=identify_main_persona(participant)) %>% unnest(cols = c(participant))

#Bring back the BotUsers column and count occurencies of each main persona
main_personas_df <- cbind(BotUsers = personas_df$BotUsers, main_personas_df)
personas_frequency <- main_personas_df %>% group_by(BotUsers,MainPersona) %>% tally(name = "Frequency")
personas_frequency


## ----fig2, out.width = '50%', out.height = '50%', fig.align = 'center'----
max_frequency <- max(personas_frequency$Frequency) #Saves highest value to set plot's scales' limits.
persona_palette <- c("None" = "#AED9D6","Sam" = "#5BB4AC", "Charlie" = "#985f99", "Alex" = "#D9B466")

fig2 <- ggplot(personas_frequency, aes(x= MainPersona, y=Frequency, fill=MainPersona))+
  geom_bar(width = 1, stat = "identity", colour = "black") + coord_flip() +
  scale_y_continuous(limits = c(0, max_frequency), breaks = seq(0,max_frequency, by=2)) +
  PLOT_THEME + scale_fill_manual(values = persona_palette) +
  theme(legend.position = "None") +
  facet_grid(personas_frequency$BotUsers ~ .)
ggsave("persona_freq.pdf", fig2, width = 14, height = 8, units = "cm")

## ----fig3, out.width = '50%', out.height = '50%', fig.align = 'center'----
#Converts to a long table format to plot density.
melted_df <- personas_df %>% gather(key="Personas", value = "Scores", -c(BotUsers))
fig3 <- ggplot(melted_df, aes(x = Scores, fill = Personas)) + geom_density() +
  scale_fill_manual(values = persona_palette) + xlim(-2,2) + PLOT_THEME + 
  theme(legend.position = "None") +
  facet_grid(rows = vars(BotUsers), cols = vars(Personas))
ggsave("density_plot.pdf", fig3, width = 16, height = 12, units = "cm")

## ----fig4, fig.width=12, fig.height=5------------------------------------

cor_charlie_sam <- ggplot(personas_df, aes(x=Charlie, y=Sam)) + geom_point() + geom_smooth(method=lm) +
  facet_grid( rows = vars(BotUsers) ) + PLOT_THEME +
  scale_y_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) +
  scale_x_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) 

cor_charlie_alex <- ggplot(personas_df, aes(x=Alex, y=Charlie)) + geom_point()+ geom_smooth(method=lm) + 
  facet_grid( rows = vars(BotUsers) ) + PLOT_THEME +
  scale_y_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) +
  scale_x_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) 

cor_sam_alex<- ggplot(personas_df, aes(x=Sam, y=Alex)) + geom_point() + geom_smooth(method=lm) +
  facet_grid( rows = vars(BotUsers) ) + PLOT_THEME +
  scale_y_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) +
  scale_x_continuous(limits = c(-2,2), breaks=seq(-2,2,by=0.5)) 

grid.arrange(cor_charlie_sam,cor_charlie_alex,cor_sam_alex, ncol = 3, top = "Correlation Between Personas")
fig4 <- arrangeGrob(cor_charlie_sam,cor_charlie_alex,cor_sam_alex, ncol = 3, top = "Correlation Between Personas")
ggsave("persona_correlations.pdf", fig4, width = 35, height = 15, units = "cm")

## ----fig5, fig.width = 7, fig.height = 5, fig.align = 'center'-----------
# Gets columns where participants scored the reasons and removes participants that never used a bot.
BOT_REASONS_COL <- c(45:54)
bot_reasons_df <- survey_answers %>% select(BOT_REASONS_COL) %>% 
  filter_all(any_vars(!is.na(.))) %>% mutate_all(factor)

# Similar to the questions text, we summarize the reasons to better plot the data. 
# The complete text for each reason can be found in the CSV data.
short_descriptions <- c("Reduce Boring Tasks", "Information Integration","Health Monitoring",
                        "Integration Quality","Common Information Gateway", "ChatOps", 
                        "Automatic Resource Cleanup", "Analyse Big Data","Problem Notifications", 
                        "Maintain Dependencies")
colnames(bot_reasons_df) <- short_descriptions

likert_reasons <- likert(bot_reasons_df)
fig5 <- plot(likert_reasons) + ggtitle("Reasons to Use a Bot") + PLOT_THEME
ggsave("bot_usage.pdf", fig5, width = 18, height = 12, units = "cm")